@@ -35,12 +35,14 @@
  * FIXME
  */
 
-#include "config.h"
-
 #include "mcd-channel.h"
 
-#include <telepathy-glib/telepathy-glib.h>
-#include <telepathy-glib/telepathy-glib-dbus.h>
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/svc-channel-request.h>
+#include <telepathy-glib/svc-generic.h>
+#include <telepathy-glib/util.h>
 
 #include "channel-utils.h"
 #include "mcd-account-priv.h"
@@ -63,6 +65,9 @@ struct _McdChannelPrivate
 
     /* boolean properties */
     guint outgoing : 1;
+    guint has_group_if : 1;
+    guint members_accepted : 1;
+    guint missed : 1;
     guint is_disposed : 1;
     guint is_aborted : 1;
     guint constructing : 1;
@@ -80,6 +85,7 @@ struct _McdChannelPrivate
 enum _McdChannelSignalType
 {
     STATUS_CHANGED,
+    MEMBERS_ACCEPTED,
     LAST_SIGNAL
 };
 
@@ -103,6 +109,67 @@ static void on_proxied_channel_status_changed (McdChannel *source,
                                                McdChannelStatus status,
                                                McdChannel *dest);
 
+static void
+on_members_changed (TpChannel *proxy, const gchar *message,
+		    const GArray *added, const GArray *removed,
+		    const GArray *l_pending, const GArray *r_pending,
+                    guint actor, guint reason, McdChannel *channel)
+{
+    McdChannelPrivate *priv = channel->priv;
+    TpHandle self_handle;
+    TpHandle conn_self_handle = 0;
+    TpHandle removed_handle = 0;
+    guint i;
+
+    self_handle = tp_channel_group_get_self_handle (proxy);
+    conn_self_handle =
+      tp_connection_get_self_handle (tp_channel_borrow_connection (proxy));
+
+    DEBUG ("called (actor %u, reason %u, self_handle %u, conn_self_handle %u)",
+           actor, reason, tp_channel_group_get_self_handle (proxy),
+           conn_self_handle);
+
+    if (added && added->len > 0)
+    {
+        DEBUG ("%u added members", added->len);
+	for (i = 0; i < added->len; i++)
+	{
+	    guint added_member = g_array_index (added, guint, i);
+            DEBUG ("added member %u", added_member);
+
+            /* see whether we are the added member */
+            if (added_member == self_handle)
+            {
+                DEBUG ("This should appear only when the call was accepted");
+                priv->members_accepted = TRUE;
+                g_signal_emit_by_name (channel, "members-accepted");
+                break;
+            }
+	}
+    }
+
+    if (removed && removed->len > 0 &&
+        (actor == 0 ||
+         reason == TP_CHANNEL_GROUP_CHANGE_REASON_ERROR ||
+         (actor != self_handle && actor != conn_self_handle) ||
+         reason == TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER))
+    {
+        for (i = 0; i < removed->len; i++)
+        {
+            removed_handle = g_array_index (removed, guint, i);
+            DEBUG ("removed member %u", removed_handle);
+            if (removed_handle == self_handle ||
+                removed_handle == conn_self_handle)
+            {
+                /* We are removed (end of call), marking as missed, if not
+                 * already accespted the call */
+                if (!priv->members_accepted) priv->missed = TRUE;
+                break;
+            }
+        }
+    }
+}
+
 static void
 proxy_destroyed (TpProxy *self, guint domain, gint code, gchar *message,
 		 gpointer user_data)
@@ -114,11 +181,22 @@ proxy_destroyed (TpProxy *self, guint domain, gint code, gchar *message,
     mcd_mission_abort (MCD_MISSION (channel));
 }
 
+static inline void
+_mcd_channel_setup_group (McdChannel *channel)
+{
+    McdChannelPrivate *priv = channel->priv;
+
+    g_signal_connect (priv->tp_chan, "group-members-changed",
+                      G_CALLBACK (on_members_changed), channel);
+}
+
 static void
 on_channel_ready (GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
     TpChannel *tp_chan = TP_CHANNEL (source_object);
     McdChannel *channel, **channel_ptr = user_data;
+    McdChannelPrivate *priv;
+    gboolean requested, valid;
     GError *error = NULL;
 
     if (!tp_proxy_prepare_finish (tp_chan, result, &error))
@@ -142,7 +220,17 @@ on_channel_ready (GObject *source_object, GAsyncResult *result, gpointer user_da
     if (!channel) return;
 
     DEBUG ("channel %p is ready", channel);
-    channel->priv->outgoing = tp_channel_get_requested (tp_chan);
+    priv = channel->priv;
+    requested = tp_asv_get_boolean
+        (tp_channel_borrow_immutable_properties (tp_chan),
+         TP_IFACE_CHANNEL ".Requested", &valid);
+    if (valid)
+        priv->outgoing = requested;
+
+    priv->has_group_if = tp_proxy_has_interface_by_id (priv->tp_chan,
+						       TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);
+    if (priv->has_group_if)
+	_mcd_channel_setup_group (channel);
 }
 
 void
@@ -196,6 +284,9 @@ _mcd_channel_release_tp_channel (McdChannel *channel)
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
     if (priv->tp_chan)
     {
+        g_signal_handlers_disconnect_by_func (G_OBJECT (priv->tp_chan),
+                                              G_CALLBACK (on_members_changed),
+                                              channel);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (priv->tp_chan),
 					      G_CALLBACK (proxy_destroyed),
 					      channel);
@@ -209,6 +300,7 @@ static void
 _mcd_channel_setup (McdChannel *channel, McdChannelPrivate *priv)
 {
     McdChannel **channel_ptr;
+    GHashTable *properties;
 
     channel_ptr = g_slice_alloc (sizeof (McdChannel *));
     *channel_ptr = channel;
@@ -218,7 +310,15 @@ _mcd_channel_setup (McdChannel *channel, McdChannelPrivate *priv)
     g_signal_connect (priv->tp_chan, "invalidated",
 		      G_CALLBACK (proxy_destroyed), channel);
 
-    priv->outgoing = tp_channel_get_requested (priv->tp_chan);
+    properties = tp_channel_borrow_immutable_properties (priv->tp_chan);
+    if (properties)
+    {
+        gboolean requested, valid = FALSE;
+        requested = tp_asv_get_boolean
+            (properties, TP_IFACE_CHANNEL ".Requested", &valid);
+        if (valid)
+            priv->outgoing = requested;
+    }
 }
 
 static void
@@ -407,7 +507,7 @@ mcd_channel_abort (McdMission *mission)
         /* this code-path can only happen if the connection is aborted, as in
          * the other cases we handle the error in McdChannel; for this reason,
          * we use the DISCONNECTED error code */
-        GError *error = g_error_new (TP_ERROR, TP_ERROR_DISCONNECTED,
+        GError *error = g_error_new (TP_ERRORS, TP_ERROR_DISCONNECTED,
                                      "Channel aborted");
         mcd_channel_take_error (channel, error);
     }
@@ -462,7 +562,7 @@ mcd_channel_status_changed (McdChannel *channel, McdChannelStatus status)
                 g_critical ("Requested channel's status changed to FAILED "
                             "without a proper error");
                 _mcd_request_set_failure (channel->priv->request,
-                                          TP_ERROR, TP_ERROR_NOT_AVAILABLE,
+                                          TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
                                           "MC bug! FAILED but no error");
             }
         }
@@ -502,6 +602,14 @@ mcd_channel_class_init (McdChannelClass * klass)
 				       status_changed_signal),
 		      NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE,
 		      1, G_TYPE_INT);
+    mcd_channel_signals[MEMBERS_ACCEPTED] =
+	g_signal_new ("members-accepted", G_OBJECT_CLASS_TYPE (klass),
+		      G_SIGNAL_RUN_FIRST,
+		      G_STRUCT_OFFSET (McdChannelClass,
+				       members_accepted_signal),
+		      NULL,
+		      NULL, g_cclosure_marshal_VOID__VOID,
+		      G_TYPE_NONE, 0);
 
     /* properties */
     g_object_class_install_property
@@ -603,10 +711,9 @@ mcd_channel_new_from_properties (TpConnection *connection,
     TpChannel *tp_chan;
     GError *error = NULL;
 
-    tp_chan = tp_simple_client_factory_ensure_channel (
-        tp_proxy_get_factory (connection), connection, object_path,
-        properties, &error);
 
+    tp_chan = tp_channel_new_from_properties (connection, object_path,
+                                              properties, &error);
     if (G_UNLIKELY (error))
     {
         g_warning ("%s: got error: %s", G_STRFUNC, error->message);
@@ -641,9 +748,9 @@ mcd_channel_new_from_path (TpConnection *connection, const gchar *object_path,
                            TpHandleType handle_type)
 {
     GHashTable *props;
-    GValue v_type = G_VALUE_INIT;
-    GValue v_handle = G_VALUE_INIT;
-    GValue v_handle_type = G_VALUE_INIT;
+    GValue v_type = { 0 };
+    GValue v_handle = { 0 };
+    GValue v_handle_type = { 0 };
     McdChannel *channel;
 
     props = g_hash_table_new (g_str_hash, g_str_equal);
@@ -689,11 +796,8 @@ _mcd_channel_create_proxy (McdChannel *channel, TpConnection *connection,
     GError *error = NULL;
 
     g_return_val_if_fail (MCD_IS_CHANNEL (channel), FALSE);
-
-    tp_chan = tp_simple_client_factory_ensure_channel (
-        tp_proxy_get_factory (connection), connection, object_path,
-        properties, &error);
-
+    tp_chan = tp_channel_new_from_properties (connection, object_path,
+                                              properties, &error);
     if (G_UNLIKELY (error))
     {
         g_warning ("%s: got error: %s", G_STRFUNC, error->message);
@@ -731,24 +835,172 @@ mcd_channel_get_status (McdChannel *channel)
     return MCD_CHANNEL_PRIV (channel)->status;
 }
 
+gboolean
+mcd_channel_get_members_accepted (McdChannel *channel)
+{
+    return MCD_CHANNEL_PRIV (channel)->members_accepted;
+}
+
+const gchar *
+mcd_channel_get_channel_type (McdChannel *channel)
+{
+    return g_quark_to_string (mcd_channel_get_channel_type_quark (channel));
+}
+
+GQuark
+mcd_channel_get_channel_type_quark (McdChannel *channel)
+{
+    McdChannelPrivate *priv;
+
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), 0);
+    priv = channel->priv;
+    if (priv->tp_chan)
+        return tp_channel_get_channel_type_id (priv->tp_chan);
+
+    if (G_LIKELY (priv->request != NULL))
+    {
+        GHashTable *properties = _mcd_request_get_properties (priv->request);
+        const gchar *type = tp_asv_get_string (properties,
+            TP_IFACE_CHANNEL ".ChannelType");
+
+        return g_quark_from_string (type);
+    }
+
+    return 0;
+}
+
 const gchar *
 mcd_channel_get_object_path (McdChannel *channel)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
 
-    return priv->tp_chan ? tp_proxy_get_object_path (priv->tp_chan) : NULL;
+    return priv->tp_chan ? TP_PROXY (priv->tp_chan)->object_path : NULL;
+}
+
+guint
+mcd_channel_get_handle (McdChannel *channel)
+{
+    McdChannelPrivate *priv;
+
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), 0);
+    priv = channel->priv;
+    if (priv->tp_chan)
+        return tp_channel_get_handle (priv->tp_chan, NULL);
+
+    if (G_LIKELY (priv->request != NULL))
+    {
+        GHashTable *properties = _mcd_request_get_properties (priv->request);
+
+        return tp_asv_get_uint32 (properties,
+            TP_IFACE_CHANNEL ".TargetHandle", NULL);
+    }
+
+    return 0;
+}
+
+TpHandleType
+mcd_channel_get_handle_type (McdChannel *channel)
+{
+    McdChannelPrivate *priv;
+    guint handle_type = TP_HANDLE_TYPE_NONE;
+
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), 0);
+    priv = channel->priv;
+    if (priv->tp_chan)
+    {
+        tp_channel_get_handle (priv->tp_chan, &handle_type);
+    }
+    else if (G_LIKELY (priv->request != NULL))
+    {
+        GHashTable *properties = _mcd_request_get_properties (priv->request);
+
+        handle_type = tp_asv_get_uint32 (properties,
+            TP_IFACE_CHANNEL ".TargetHandle", NULL);
+    }
+
+    return handle_type;
+}
+
+/**
+ * mcd_channel_get_name:
+ * @channel: the #McdChannel.
+ *
+ * Get the Telepathy name of @channel (calls InspectHandles on the channel
+ * handle).
+ *
+ * Returns: a const string holding the channel name.
+ */
+const gchar *
+mcd_channel_get_name (McdChannel *channel)
+{
+    McdChannelPrivate *priv;
+    GHashTable *properties = NULL;
+
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), NULL);
+    priv = channel->priv;
+
+    if (priv->tp_chan)
+        properties = tp_channel_borrow_immutable_properties (priv->tp_chan);
+    else if (G_LIKELY (priv->request != NULL))
+        properties = _mcd_request_get_properties (priv->request);
+
+    if (!properties) return NULL;
+
+    return tp_asv_get_string (properties, TP_IFACE_CHANNEL ".TargetID");
+}
+
+/**
+ * mcd_channel_get_inviter:
+ * @channel: the #McdChannel.
+ *
+ * Get the address of the inviter (i.e. the actor who put us in the pending
+ * local members list).
+ *
+ * Returns: a const string holding the inviter address.
+ */
+const gchar *
+mcd_channel_get_inviter (McdChannel *channel)
+{
+    McdChannelPrivate *priv;
+    GHashTable *properties = NULL;
+
+    g_return_val_if_fail (MCD_IS_CHANNEL (channel), NULL);
+    priv = channel->priv;
+    if (priv->tp_chan)
+    {
+        properties = tp_channel_borrow_immutable_properties (priv->tp_chan);
+        if (properties)
+            return tp_asv_get_string (properties,
+                                      TP_IFACE_CHANNEL ".InitiatorID");
+    }
+    return NULL;
+}
+
+/**
+ * mcd_channel_is_missed:
+ * @channel: the #McdChannel.
+ *
+ * Return %TRUE if the remote party removed itself before we could join the
+ * channel.
+ *
+ * Returns: %TRUE if the channel is missed.
+ */
+gboolean
+mcd_channel_is_missed (McdChannel *channel)
+{
+    return MCD_CHANNEL_PRIV (channel)->missed;
 }
 
 /*
- * mcd_channel_dup_immutable_properties:
+ * _mcd_channel_get_immutable_properties:
  * @channel: the #McdChannel.
  *
- * Returns: the %G_VARIANT_TYPE_VARDICT of the immutable properties.
+ * Returns: the #GHashTable of the immutable properties.
  */
-GVariant *
-mcd_channel_dup_immutable_properties (McdChannel *channel)
+GHashTable *
+_mcd_channel_get_immutable_properties (McdChannel *channel)
 {
-    GVariant *ret;
+    GHashTable *ret;
 
     g_return_val_if_fail (MCD_IS_CHANNEL (channel), NULL);
 
@@ -758,7 +1010,7 @@ mcd_channel_dup_immutable_properties (McdChannel *channel)
         return NULL;
     }
 
-    ret = tp_channel_dup_immutable_properties (channel->priv->tp_chan);
+    ret = tp_channel_borrow_immutable_properties (channel->priv->tp_chan);
 
     if (ret == NULL)
     {
@@ -812,7 +1064,7 @@ mcd_channel_get_error (McdChannel *channel)
         return priv->error;
 
     if (priv->tp_chan)
-        return tp_proxy_get_invalidated (priv->tp_chan);
+        return TP_PROXY (priv->tp_chan)->invalidated;
 
     return NULL;
 }
@@ -826,7 +1078,7 @@ _mcd_channel_request_cancelling_cb (McdRequest *request,
     g_object_ref (self);
     DEBUG ("%p in status %u", self, status);
 
-    mcd_channel_take_error (self, g_error_new (TP_ERROR,
+    mcd_channel_take_error (self, g_error_new (TP_ERRORS,
                                                TP_ERROR_CANCELLED,
                                                "Cancelled"));
 
@@ -1124,22 +1376,20 @@ mcd_channel_get_tp_channel (McdChannel *channel)
 }
 
 static void
-mcd_channel_depart_cb (GObject *source_object,
-                       GAsyncResult *result,
-                       gpointer data G_GNUC_UNUSED)
+mcd_channel_depart_cb (TpChannel *channel,
+                       const GError *error,
+                       gpointer data G_GNUC_UNUSED,
+                       GObject *weak_object G_GNUC_UNUSED)
 {
-    GError *error = NULL;
-
-    /* By this point, TpChannel has already called Close() for us;
-     * we only get an error if that failed. If Close() fails, there's
-     * not a whole lot we can do about it. */
-
-    if (!tp_channel_leave_finish (TP_CHANNEL (source_object), result, &error))
+    if (error == NULL)
     {
-        WARNING ("failed to depart, even via Close(): %s %d: %s",
-               g_quark_to_string (error->domain), error->code, error->message);
-        g_error_free (error);
+        DEBUG ("successful");
+        return;
     }
+
+    DEBUG ("failed to depart, calling Close instead: %s %d: %s",
+           g_quark_to_string (error->domain), error->code, error->message);
+    tp_cli_channel_call_close (channel, -1, NULL, NULL, NULL, NULL);
 }
 
 typedef struct {
@@ -1166,10 +1416,22 @@ mcd_channel_ready_to_depart_cb (GObject *source_object,
         return;
     }
 
-    /* If it's a Group, this will leave gracefully.
-     * If not, it will just close it. Either's good. */
-    tp_channel_leave_async (channel, d->reason, d->message,
-                            mcd_channel_depart_cb, NULL);
+    if (tp_proxy_has_interface_by_id (channel,
+                                      TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP))
+    {
+        GArray *a = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
+        guint self_handle = tp_channel_group_get_self_handle (channel);
+
+        g_array_append_val (a, self_handle);
+
+        tp_cli_channel_interface_group_call_remove_members_with_reason (
+            channel, -1, a, d->message, d->reason,
+            mcd_channel_depart_cb, NULL, NULL, NULL);
+
+        g_array_unref (a);
+        g_free (d->message);
+        g_slice_free (DepartData, d);
+    }
 }
 
 void
@@ -1179,7 +1441,6 @@ _mcd_channel_depart (McdChannel *channel,
 {
     DepartData *d;
     const GError *invalidated;
-    GQuark just_group_feature[] = { TP_CHANNEL_FEATURE_GROUP, 0 };
 
     g_return_if_fail (MCD_IS_CHANNEL (channel));
 
@@ -1198,7 +1459,8 @@ _mcd_channel_depart (McdChannel *channel,
     if (message[0] == '\0' && reason == TP_CHANNEL_GROUP_CHANGE_REASON_NONE)
     {
         /* exactly equivalent to Close(), so skip the Group interface */
-        tp_channel_close_async (channel->priv->tp_chan, NULL, NULL);
+        tp_cli_channel_call_close (channel->priv->tp_chan, -1,
+                                   NULL, NULL, NULL, NULL);
         return;
     }
 
@@ -1206,9 +1468,7 @@ _mcd_channel_depart (McdChannel *channel,
     d->reason = reason;
     d->message = g_strdup (message);
 
-    /* tp_channel_leave_async documents calling it without first preparing
-     * GROUP as deprecated. */
-    tp_proxy_prepare_async (channel->priv->tp_chan, just_group_feature,
+    tp_proxy_prepare_async (channel->priv->tp_chan, NULL,
                             mcd_channel_ready_to_depart_cb, d);
 }
 
