@@ -29,7 +29,8 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <telepathy-glib/telepathy-glib.h>
+#include <telepathy-glib/channel.h>
+#include <telepathy-glib/dbus-properties-mixin.h>
 
 #include "mcd-mission.h"
 
@@ -79,6 +80,13 @@ struct _McdChannelClass
     /* signals */
     void (*status_changed_signal) (McdChannel * channel,
 				   McdChannelStatus status);
+    void (*members_accepted_signal) (McdChannel *channel);
+    TpDBusPropertiesMixinClass _former_dbus_properties_class;
+    void (*_mc_reserved1) (void);
+    void (*_mc_reserved2) (void);
+    void (*_mc_reserved3) (void);
+    void (*_mc_reserved4) (void);
+    void (*_mc_reserved5) (void);
 };
 
 GType mcd_channel_get_type (void);
@@ -93,7 +101,15 @@ McdChannel *mcd_channel_new_from_path (TpConnection *connection,
                                        TpHandleType handle_type);
 
 McdChannelStatus mcd_channel_get_status (McdChannel * channel);
+gboolean mcd_channel_get_members_accepted (McdChannel *channel);
+const gchar* mcd_channel_get_channel_type (McdChannel *channel);
+GQuark mcd_channel_get_channel_type_quark (McdChannel *channel);
 const gchar* mcd_channel_get_object_path (McdChannel *channel);
+guint mcd_channel_get_handle (McdChannel *channel);
+TpHandleType mcd_channel_get_handle_type (McdChannel *channel);
+const gchar *mcd_channel_get_name (McdChannel *channel);
+const gchar *mcd_channel_get_inviter (McdChannel *channel);
+gboolean mcd_channel_is_missed (McdChannel *channel);
 gboolean mcd_channel_is_requested (McdChannel *channel);
 McdAccount *mcd_channel_get_account (McdChannel *channel);
 TpChannel *mcd_channel_get_tp_channel (McdChannel *channel);
@@ -101,7 +117,5 @@ TpChannel *mcd_channel_get_tp_channel (McdChannel *channel);
 void mcd_channel_take_error (McdChannel *channel, GError *error);
 const GError *mcd_channel_get_error (McdChannel *channel);
 
-GVariant *mcd_channel_dup_immutable_properties (McdChannel *channel);
-
 G_END_DECLS
 #endif /* MCD_CHANNEL_H */
