@@ -41,7 +41,6 @@
 #include <sched.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
 #include <string.h>
@@ -55,10 +54,6 @@
 #include <telepathy-glib/proxy-subclass.h>
 #include <telepathy-glib/util.h>
 
-#include <libmcclient/mc-errors.h>
-#include <libmcclient/mc-gtypes.h>
-#include <libmcclient/mc-interfaces.h>
-
 #include "mcd-account-priv.h"
 #include "mcd-channel-priv.h"
 #include "mcd-connection-priv.h"
@@ -70,12 +65,6 @@
 #include "sp_timestamp.h"
 
 #include "mcd-signals-marshal.h"
-#include "_gen/cli-Connection_Interface_Contact_Capabilities_Draft1.h"
-#include "_gen/cli-Connection_Interface_Contact_Capabilities.h"
-#include "_gen/cli-Connection_Interface_Contact_Capabilities_Draft1-body.h"
-#include "_gen/cli-Connection_Interface_Contact_Capabilities-body.h"
-#include "_gen/cli-Connection_Interface_Power_Saving.h"
-#include "_gen/cli-Connection_Interface_Power_Saving-body.h"
 
 #define INITIAL_RECONNECTION_TIME   3 /* seconds */
 #define RECONNECTION_MULTIPLIER     3
@@ -97,9 +86,6 @@ struct _McdConnectionPrivate
     /* Account */
     McdAccount *account;
 
-    /* Associated profile */
-    /* McProfile *profile; */
-
     /* Telepathy connection manager */
     TpConnectionManager *tp_conn_mgr;
 
@@ -124,7 +110,6 @@ struct _McdConnectionPrivate
     guint has_avatars_if : 1;
     guint has_alias_if : 1;
     guint has_capabilities_if : 1;
-    guint has_contact_capabilities_draft1_if : 1;
     guint has_contact_capabilities_if : 1;
     guint has_power_saving_if : 1;
 
@@ -423,7 +408,7 @@ presence_get_statuses_cb (TpProxy *proxy, const GValue *v_statuses,
     }
 
     /* Now the presence info is ready. We can set the presence */
-    _mcd_account_get_requested_presence (priv->account, &presence,
+    mcd_account_get_requested_presence (priv->account, &presence,
                                          &status, &message);
     if (priv->connected)
     {
@@ -491,7 +476,7 @@ _mcd_connection_setup_presence (McdConnection *connection)
         (priv->tp_conn, -1, self_handle_array,
          mcd_connection_initial_presence_cb, priv, NULL,
          (GObject *) connection);
-    g_array_free (self_handle_array, TRUE);
+    g_array_unref (self_handle_array);
 
     tp_cli_dbus_properties_call_get
         (priv->tp_conn, -1, TP_IFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE,
@@ -652,32 +637,7 @@ _mcd_connection_setup_capabilities (McdConnection *connection)
 				   G_TYPE_UINT, G_TYPE_INVALID);
     for (i = 0; i < capabilities->len; i++)
 	g_boxed_free (type, g_ptr_array_index (capabilities, i));
-    g_ptr_array_free (capabilities, TRUE);
-}
-
-static void
-_mcd_connection_setup_contact_capabilities (McdConnection *connection)
-{
-    McdConnectionPrivate *priv = MCD_CONNECTION_PRIV (connection);
-    GPtrArray *contact_capabilities;
-
-    if (!priv->has_contact_capabilities_draft1_if)
-    {
-        DEBUG ("connection does not support contact capabilities interface");
-	priv->got_contact_capabilities = TRUE;
-	return;
-    }
-    contact_capabilities = _mcd_dispatcher_get_channel_enhanced_capabilities
-      (priv->dispatcher);
-
-    DEBUG ("advertising capabilities");
-
-    mc_cli_connection_interface_contact_capabilities_draft1_call_set_self_capabilities
-      (priv->tp_conn, -1, contact_capabilities, NULL, NULL, NULL, NULL);
-    DEBUG ("SetSelfCapabilities: Called.");
-
-    /* free the connection capabilities (FIXME) */
-    g_ptr_array_free (contact_capabilities, TRUE);
+    g_ptr_array_unref (capabilities);
 }
 
 static void
@@ -888,7 +848,7 @@ avatars_request_tokens_cb (TpConnection *proxy, GHashTable *tokens,
     {
         DEBUG ("No avatar set, setting our own");
         _mcd_connection_set_avatar (connection, avatar, mime_type);
-        g_array_free (avatar, TRUE);
+        g_array_unref (avatar);
     }
     g_free (mime_type);
 }
@@ -939,7 +899,7 @@ _mcd_connection_setup_avatar (McdConnection *connection)
 									      priv, NULL,
 									      (GObject *)connection);
 	}
-	g_array_free (avatar, TRUE);
+	g_array_unref (avatar);
     }
     g_free (mime_type);
 }
@@ -1011,7 +971,7 @@ _mcd_connection_set_nickname (McdConnection *connection,
 							   aliasing_set_aliases_cb,
 							   priv, NULL,
 							   (GObject *)connection);
-    g_hash_table_destroy (aliases);
+    g_hash_table_unref (aliases);
 }
 
 static void
@@ -1067,7 +1027,7 @@ _mcd_connection_setup_alias (McdConnection *connection)
     tp_cli_connection_interface_aliasing_call_get_aliases
         (priv->tp_conn, -1, self_handle_array, _mcd_connection_get_aliases_cb,
          priv, NULL, (GObject *) connection);
-    g_array_free (self_handle_array, TRUE);
+    g_array_unref (self_handle_array);
 }
 
 static void
@@ -1081,7 +1041,7 @@ _mcd_connection_setup_power_saving (McdConnection *connection)
   DEBUG ("is %sactive", mcd_slacker_is_inactive (priv->slacker) ? "in" : "");
 
   if (mcd_slacker_is_inactive (priv->slacker))
-    mc_cli_connection_interface_power_saving_call_set_power_saving (priv->tp_conn, -1,
+    tp_cli_connection_interface_power_saving_call_set_power_saving (priv->tp_conn, -1,
         TRUE, NULL, NULL, NULL, NULL);
 }
 
@@ -1581,7 +1541,7 @@ list_channels_cb (TpConnection *connection,
         g_hash_table_insert (channel_props, TP_IFACE_CHANNEL ".TargetHandle",
                              va->values + 3);
         mcd_connection_found_channel (self, object_path, channel_props);
-        g_hash_table_destroy (channel_props);
+        g_hash_table_unref (channel_props);
     }
 
     self->priv->dispatched_initial_channels = TRUE;
@@ -1601,21 +1561,25 @@ mcd_connection_setup_pre_requests (McdConnection *connection)
 }
 
 static void
-on_connection_ready (TpConnection *tp_conn, const GError *error,
-		     gpointer user_data)
+on_connection_ready (GObject *source_object, GAsyncResult *result,
+                     gpointer user_data)
 {
+    TpConnection *tp_conn = TP_CONNECTION (source_object);
     McdConnection *connection, **connection_ptr = user_data;
     McdConnectionPrivate *priv;
+    GError *error = NULL;
 
     connection = *connection_ptr;
     if (connection)
 	g_object_remove_weak_pointer ((GObject *)connection,
 				      (gpointer)connection_ptr);
     g_slice_free (McdConnection *, connection_ptr);
-    if (error)
+
+    if (!tp_proxy_prepare_finish (tp_conn, result, &error))
     {
         DEBUG ("got error: %s", error->message);
-	return;
+        g_clear_error (&error);
+        return;
     }
 
     if (!connection) return;
@@ -1631,12 +1595,10 @@ on_connection_ready (TpConnection *tp_conn, const GError *error,
 						       TP_IFACE_QUARK_CONNECTION_INTERFACE_ALIASING);
     priv->has_capabilities_if = tp_proxy_has_interface_by_id (tp_conn,
 							      TP_IFACE_QUARK_CONNECTION_INTERFACE_CAPABILITIES);
-    priv->has_contact_capabilities_draft1_if = tp_proxy_has_interface_by_id (tp_conn,
-        MC_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_CAPABILITIES_DRAFT1);
     priv->has_contact_capabilities_if = tp_proxy_has_interface_by_id (tp_conn,
         TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_CAPABILITIES);
     priv->has_power_saving_if = tp_proxy_has_interface_by_id (tp_conn,
-        MC_IFACE_QUARK_CONNECTION_INTERFACE_POWER_SAVING);
+        TP_IFACE_QUARK_CONNECTION_INTERFACE_POWER_SAVING);
 
     if (priv->has_presence_if)
 	_mcd_connection_setup_presence (connection);
@@ -1644,9 +1606,6 @@ on_connection_ready (TpConnection *tp_conn, const GError *error,
     if (priv->has_capabilities_if)
 	_mcd_connection_setup_capabilities (connection);
 
-    if (priv->has_contact_capabilities_draft1_if)
-	_mcd_connection_setup_contact_capabilities (connection);
-
     if (priv->has_avatars_if)
 	_mcd_connection_setup_avatar (connection);
 
@@ -1832,7 +1791,7 @@ mcd_connection_early_get_interfaces_cb (TpConnection *tp_conn,
                     _mcd_connection_update_client_caps (self, client_caps);
                     g_ptr_array_foreach (client_caps,
                                          (GFunc) g_value_array_free, NULL);
-                    g_ptr_array_free (client_caps, TRUE);
+                    g_ptr_array_unref (client_caps);
                 }
                 /* else the McdDispatcher hasn't sorted itself out yet, so
                  * we can't usefully pre-load capabilities - we'll be told
@@ -1994,7 +1953,7 @@ _mcd_connection_finalize (GObject * object)
 
     g_free (priv->alias);
     if (priv->recognized_presences)
-        g_hash_table_destroy (priv->recognized_presences);
+        g_hash_table_unref (priv->recognized_presences);
 
     G_OBJECT_CLASS (mcd_connection_parent_class)->finalize (object);
 }
@@ -2063,7 +2022,7 @@ on_inactivity_changed (McdSlacker *slacker,
       priv->has_power_saving_if ? "has" : "doesn't");
 
   if (priv->has_power_saving_if)
-    mc_cli_connection_interface_power_saving_call_set_power_saving (priv->tp_conn, -1,
+    tp_cli_connection_interface_power_saving_call_set_power_saving (priv->tp_conn, -1,
         inactive, NULL, NULL, NULL, NULL);
 }
 
@@ -2333,7 +2292,7 @@ _mcd_connection_request_channel (McdConnection *connection, McdChannel *channel)
     g_return_val_if_fail (priv->tp_conn != NULL, FALSE);
     g_return_val_if_fail (TP_IS_CONNECTION (priv->tp_conn), FALSE);
 
-    if (!tp_connection_is_ready (priv->tp_conn))
+    if (!tp_proxy_is_prepared (priv->tp_conn, TP_CONNECTION_FEATURE_CONNECTED))
     {
         /* don't request any channel until the connection is ready (because we
          * don't know if the CM implements the Requests interface). The channel
@@ -2361,22 +2320,6 @@ _mcd_connection_request_channel (McdConnection *connection, McdChannel *channel)
     return ret;
 }
 
-static void
-mcd_connection_add_signals (TpProxy *self,
-                            guint quark,
-                            DBusGProxy *proxy,
-                            gpointer data)
-{
-    mc_cli_Connection_Interface_Contact_Capabilities_Draft1_add_signals (self,
-                                                                         quark,
-                                                                         proxy,
-                                                                         data);
-    mc_cli_Connection_Interface_Contact_Capabilities_add_signals (self, quark,
-                                                                  proxy, data);
-    mc_cli_Connection_Interface_Power_Saving_add_signals (self, quark,
-        proxy, data);
-}
-
 static void
 mcd_connection_class_init (McdConnectionClass * klass)
 {
@@ -2395,8 +2338,6 @@ mcd_connection_class_init (McdConnectionClass * klass)
     _mcd_ext_register_dbus_glib_marshallers ();
 
     tp_connection_init_known_interfaces ();
-    tp_proxy_or_subclass_hook_on_interface_add (TP_TYPE_CONNECTION,
-                                                mcd_connection_add_signals);
 
     /* Properties */
     g_object_class_install_property
@@ -2779,6 +2720,10 @@ _mcd_connection_set_tp_connection (McdConnection *connection,
 {
     McdConnection **connection_ptr;
     McdConnectionPrivate *priv;
+    GQuark features[] = {
+      TP_CONNECTION_FEATURE_CONNECTED,
+      0
+    };
 
     g_return_if_fail (MCD_IS_CONNECTION (connection));
     priv = connection->priv;
@@ -2826,8 +2771,8 @@ _mcd_connection_set_tp_connection (McdConnection *connection,
     *connection_ptr = connection;
     g_object_add_weak_pointer ((GObject *)connection,
                                (gpointer)connection_ptr);
-    tp_connection_call_when_ready (priv->tp_conn, on_connection_ready,
-                                   connection_ptr);
+    tp_proxy_prepare_async (priv->tp_conn, features,
+                            on_connection_ready, connection_ptr);
 }
 
 /**
@@ -2849,7 +2794,7 @@ _mcd_connection_is_ready (McdConnection *self)
     g_return_val_if_fail (MCD_IS_CONNECTION (self), FALSE);
 
     return (self->priv->tp_conn != NULL) &&
-        tp_connection_is_ready (self->priv->tp_conn);
+        tp_proxy_is_prepared (self->priv->tp_conn, TP_CONNECTION_FEATURE_CONNECTED);
 }
 
 gboolean
