@@ -58,7 +58,6 @@ struct _McdManagerPrivate
 {
     gchar *name;
     TpDBusDaemon *dbus_daemon;
-    TpSimpleClientFactory *client_factory;
     McdDispatcher *dispatcher;
 
     TpConnectionManager *tp_conn_mgr;
@@ -74,7 +73,7 @@ enum
     PROP_0,
     PROP_NAME,
     PROP_DISPATCHER,
-    PROP_CLIENT_FACTORY
+    PROP_DBUS_DAEMON,
 };
 
 static GQuark readiness_quark = 0;
@@ -97,6 +96,35 @@ on_manager_ready (GObject *source_object,
     g_clear_error (&error);
 }
 
+static gint
+_find_connection_by_path (gconstpointer data, gconstpointer user_data)
+{
+    TpConnection *tp_conn;
+    McdConnection *connection = MCD_CONNECTION (data);
+    const gchar *object_path = (const gchar *)user_data;
+    const gchar *conn_object_path = NULL;
+    gint ret;
+
+    if (!data) return 1;
+
+    g_object_get (G_OBJECT (connection), "tp-connection",
+		  &tp_conn, NULL);
+    if (!tp_conn)
+	return 1;
+    conn_object_path = TP_PROXY (tp_conn)->object_path;
+    if (strcmp (conn_object_path, object_path) == 0)
+    {
+	ret = 0;
+    }
+    else
+    {
+	ret = 1;
+    }
+    
+    g_object_unref (G_OBJECT (tp_conn));
+    return ret;
+}
+
 static void
 _mcd_manager_finalize (GObject * object)
 {
@@ -123,7 +151,6 @@ _mcd_manager_dispose (GObject * object)
 
     tp_clear_object (&priv->dispatcher);
     tp_clear_object (&priv->tp_conn_mgr);
-    tp_clear_object (&priv->client_factory);
     tp_clear_object (&priv->dbus_daemon);
     tp_clear_object (&priv->slacker);
 
@@ -231,14 +258,10 @@ _mcd_manager_set_property (GObject * obj, guint prop_id,
 	tp_clear_object (&priv->dispatcher);
 	priv->dispatcher = dispatcher;
 	break;
-
-    case PROP_CLIENT_FACTORY:
-        g_assert (priv->client_factory == NULL); /* construct-only */
-        priv->client_factory = g_value_dup_object (val);
-        priv->dbus_daemon = g_object_ref (
-            tp_simple_client_factory_get_dbus_daemon (priv->client_factory));
-        break;
-
+    case PROP_DBUS_DAEMON:
+	tp_clear_object (&priv->dbus_daemon);
+	priv->dbus_daemon = TP_DBUS_DAEMON (g_value_dup_object (val));
+	break;
     default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 	break;
@@ -256,12 +279,29 @@ _mcd_manager_get_property (GObject * obj, guint prop_id,
     case PROP_DISPATCHER:
 	g_value_set_object (val, priv->dispatcher);
 	break;
+    case PROP_DBUS_DAEMON:
+	g_value_set_object (val, priv->dbus_daemon);
+	break;
     default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 	break;
     }
 }
 
+static McdConnection *
+create_connection (McdManager *manager, McdAccount *account)
+{
+    McdManagerPrivate *priv = manager->priv;
+
+    return g_object_new (MCD_TYPE_CONNECTION,
+                         "dbus-daemon", priv->dbus_daemon,
+                         "tp-manager", priv->tp_conn_mgr,
+                         "dispatcher", priv->dispatcher,
+                         "account", account,
+                         "slacker", priv->slacker,
+                         NULL);
+}
+
 static void
 mcd_manager_class_init (McdManagerClass * klass)
 {
@@ -279,6 +319,8 @@ mcd_manager_class_init (McdManagerClass * klass)
     mission_class->connect = _mcd_manager_connect;
     mission_class->disconnect = _mcd_manager_disconnect;
 
+    klass->create_connection = create_connection;
+
     /* Properties */
     g_object_class_install_property
         (object_class, PROP_NAME,
@@ -292,11 +334,11 @@ mcd_manager_class_init (McdManagerClass * klass)
                               "Dispatcher",
                               MCD_TYPE_DISPATCHER,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-    g_object_class_install_property (object_class, PROP_CLIENT_FACTORY,
-        g_param_spec_object ("client-factory", "Client factory",
-            "Client factory", TP_TYPE_SIMPLE_CLIENT_FACTORY,
-            G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+    g_object_class_install_property
+        (object_class, PROP_DBUS_DAEMON,
+         g_param_spec_object ("dbus-daemon", "DBus daemon", "DBus daemon",
+                              TP_TYPE_DBUS_DAEMON,
+                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
     readiness_quark = g_quark_from_static_string ("mcd_manager_got_info");
 }
@@ -316,17 +358,58 @@ mcd_manager_init (McdManager *manager)
 McdManager *
 mcd_manager_new (const gchar *unique_name,
 		 McdDispatcher *dispatcher,
-                 TpSimpleClientFactory *client_factory)
+		 TpDBusDaemon *dbus_daemon)
 {
     McdManager *obj;
     obj = MCD_MANAGER (g_object_new (MCD_TYPE_MANAGER,
 				     "name", unique_name,
 				     "dispatcher", dispatcher,
-                                     "client-factory", client_factory,
-                                     NULL));
+				     "dbus-daemon", dbus_daemon, NULL));
     return obj;
 }
 
+McdConnection *
+mcd_manager_get_connection (McdManager * manager, const gchar *object_path)
+{
+    const GList *connections;
+    const GList *node;
+
+    connections = mcd_operation_get_missions (MCD_OPERATION (manager));
+    node = g_list_find_custom ((GList*)connections, object_path,
+			       _find_connection_by_path);
+
+    if (node != NULL)
+    {
+	return MCD_CONNECTION (node->data);
+    }
+
+    else
+    {
+	return NULL;
+    }
+}
+
+gboolean
+mcd_manager_cancel_channel_request (McdManager *manager, guint operation_id,
+				    const gchar *requestor_client_id,
+				    GError **error)
+{
+    const GList *connections, *node;
+
+    connections = mcd_operation_get_missions (MCD_OPERATION (manager));
+    if (!connections) return FALSE;
+
+    for (node = connections; node; node = node->next)
+    {
+	if (mcd_connection_cancel_channel_request (MCD_CONNECTION (node->data),
+						   operation_id,
+						   requestor_client_id,
+						   error))
+	    return TRUE;
+    }
+    return FALSE;
+}
+
 /**
  * mcd_manager_get_unique_name:
  * @manager: the #McdManager.
@@ -342,21 +425,21 @@ mcd_manager_get_name (McdManager *manager)
     return priv->name;
 }
 
-TpProtocol *
+TpConnectionManagerProtocol *
 _mcd_manager_dup_protocol (McdManager *manager,
                            const gchar *protocol)
 {
-    TpProtocol *p;
+    const TpConnectionManagerProtocol *p;
     g_return_val_if_fail (MCD_IS_MANAGER (manager), NULL);
     g_return_val_if_fail (protocol != NULL, NULL);
 
-    p = tp_connection_manager_get_protocol_object (manager->priv->tp_conn_mgr,
-                                                   protocol);
+    p = tp_connection_manager_get_protocol (manager->priv->tp_conn_mgr,
+                                            protocol);
 
     if (p == NULL)
         return NULL;
     else
-        return g_object_ref (p);
+        return tp_connection_manager_protocol_copy (p);
 }
 
 const TpConnectionManagerParam *
@@ -364,7 +447,7 @@ mcd_manager_get_protocol_param (McdManager *manager, const gchar *protocol,
                                 const gchar *param)
 {
     McdManagerPrivate *priv;
-    TpProtocol *cm_protocol;
+    const TpConnectionManagerProtocol *cm_protocol;
 
     g_return_val_if_fail (MCD_IS_MANAGER (manager), NULL);
     g_return_val_if_fail (protocol != NULL, NULL);
@@ -372,13 +455,13 @@ mcd_manager_get_protocol_param (McdManager *manager, const gchar *protocol,
 
     priv = manager->priv;
 
-    cm_protocol = tp_connection_manager_get_protocol_object (priv->tp_conn_mgr,
-                                                             protocol);
+    cm_protocol = tp_connection_manager_get_protocol (priv->tp_conn_mgr,
+                                                      protocol);
 
     if (cm_protocol == NULL)
         return NULL;
 
-    return tp_protocol_get_param (cm_protocol, param);
+    return tp_connection_manager_protocol_get_param (cm_protocol, param);
 }
 
 McdConnection *
@@ -389,14 +472,8 @@ mcd_manager_create_connection (McdManager *manager, McdAccount *account)
     g_return_val_if_fail (MCD_IS_MANAGER (manager), NULL);
     g_return_val_if_fail (manager->priv->tp_conn_mgr != NULL, NULL);
 
-    connection = g_object_new (MCD_TYPE_CONNECTION,
-                               "client-factory", manager->priv->client_factory,
-                               "tp-manager", manager->priv->tp_conn_mgr,
-                               "dispatcher", manager->priv->dispatcher,
-                               "account", account,
-                               "slacker", manager->priv->slacker,
-                               NULL);
-
+    connection = MCD_MANAGER_GET_CLASS (manager)->create_connection
+        (manager, account);
     mcd_operation_take_mission (MCD_OPERATION (manager),
 				MCD_MISSION (connection));
     DEBUG ("Created a connection %p for account: %s",
@@ -418,6 +495,19 @@ mcd_manager_get_tp_proxy (McdManager *manager)
     return manager->priv->tp_conn_mgr;
 }
 
+/**
+ * mcd_manager_get_dispatcher:
+ * @manager: the #McdManager.
+ *
+ * Returns: the #McdDispatcher.
+ */
+McdDispatcher *
+mcd_manager_get_dispatcher (McdManager *manager)
+{
+    g_return_val_if_fail (MCD_IS_MANAGER (manager), NULL);
+    return manager->priv->dispatcher;
+}
+
 /**
  * mcd_manager_call_when_ready:
  * @manager: the #McdManager.
